==> eh-0.4.tar.gz: eh-0.4/src/ehperror.c <==
/***

ehperror.c -- Error Handler, print error messages

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehperror.c,v 1.16 1996/05/12 15:48:30 ccobb Exp $";

/***

PrLNxSerror messages to the error log file.
Error messages are stored in error message files
having a name of the form:

   IDerrmsg.txt

These error message text files are genereated from an
error message "source" file with the utility ehdivsrc 
(error handler: divide source).  This utility takes an
error message source file and divides it into an error
message text file and a header file of the same name. 
The header file contains #defined ID's for each error 
message.  The appropriate ID must be passed to [UhS
EHTHROW(iErrorID) macro, which causes the appropriate 
error message to be fetched from the error message file
and written the error log file.

ehPError(iErrorID)

***/

#include "eh.h"

EHBEGINEXTERNC

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <time.h>

#if SUPPORTS_STDARG
#include <stdarg.h>
#else 
#include <varargs.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

EHENDEXTERNC

#include "ehchnvol.h"
#include "ehfuntrc.h"
#include "eherrlog.h"
//#include "ehpgmtrc.h"
#include "ehdivsrc.h"
#include "ehperror.h"
#include "ehintini.h"
#include "ehercode.h"

static bool isErrorMsg A((char *pcMsg));

/***

ehPError()

Log error message according to error id.

ehPError(__FILE__, __FUNC__, __LINE__, scIsFatal, ulStampOff, Er);

CAUTION: This function may not return the SAME error code that was
         passed to it!  The reason for this is so the earliest/
         most severe error code is propogated to the completion
         of the program.  Therefore, if a severe error has already
         occurred and Eh.Er.u.hNo represents that fact, and this
         function is subsequently called with a less severe error
         code, then the more severe value of Eh.Er.u.hNo will be
         returned from this function rather than the error code
         that is passed in!

***/ 

void
ehPError A6(const char *, pcFilNam,
            const char *, pcFunNam,
            int,          iLineNum,
            schar,        scIsFatal, /* 1 = non=fatal, -1 = fatal */
            ulong,        ulStampOff,
            ulong,        ulHash)    /* "hashed" error code from error file */
{
   ER Er;
   int   iSavErrNo = errno;

   Er = ehUnHash(ulHash);

   //printf("ehPError: ulHash = 0x%lx, Er is:\n", ulHash);
   //dmpER(Er);

   ehLogTxt("\n");   /* spit out system header stuff */

   /* output the error ID and location */
   ehIndent(EHSTDERR);
   fprintf(EHSTDERR, "error %s-%04x <%s %s%s%d>\n",
      Er.acPrefix, Er.hCode,
      pcFilNam,
      pcFunNam ? pcFunNam : "",
      pcFunNam ? "() "    : "",
      iLineNum);
   fflush(EHSTDERR);

   /* output the program name, error symbol & text */
   ehFndMsg(ulStampOff, Er); /* find message and send to stderr */

   if (iSavErrNo)
   {
      ehIndent(EHSTDERR);
      errno = iSavErrNo;
      fprintf(EHSTDERR, "errno: %s\n", strerror(errno));
      errno = 0;
   }
   ehCallStack(EHSTDERR);

   /*** now determine which error code to return ***/
   /*** return the earliest/most severe (first/worst) error ***/

   /* preserve the earliest FATAL error */;
   EhSt.scError = scIsFatal;  /* an error occurred in the current function */
   if (Eh.Er.hCode >= 0) /* a previous NON-fatal error */
   {
      strcpy(Eh.Er.acPrefix, Er.acPrefix);
      if (scIsFatal == -1) /* override previous NON-fatal error w fatal error */
      {
         Eh.Er.hCode = (short)-Er.hCode; /* set to fatal error */
      }
      else if (Eh.Er.hCode == 0)  /* don't over-write previous error */
      {
         Eh.Er.hCode =  Er.hCode;
      }
   }
   //printf("ehPError: Eh.Er.lNo   = 0x%x\n", Eh.Er.lNo);
   //printf("ehPError: Eh.Er.u.c0  = %c\n",   Eh.Er.u.c0);
   //printf("ehPError: Eh.Er.u.c1  = %c\n",   Eh.Er.u.c1);
   //printf("ehPError: Eh.Er.u.hNo = %d\n",   Eh.Er.u.hNo);
   if (scIsFatal == -1)
   {
      EhSt.yChain = 1;   /* set the chaining bit */

      /***
         NOTE:
         The following are NOT guarenteed to
         cause a longjmp or throw.  We may not
         be allowed to jump/throw now, in which
         case we will just return.
      ***/

#ifdef ehEXCEPTIONHANDLING

      ehPErrorThrow(pcFunNam);   /* throw ! */

#else /* !defined(__cplusplus) || !ehEXCEPTIONHANDLING */

      ehFatalJmp();              /* longjmp() ! */

#endif /* !defined(__cplusplus) | !ehEXCEPTIONHANDLING */

   }
   return;
} /* ehPError */


/***
ehFndMsg()
Error Handler -- Find error message and send to stderr.
Terminate with a newline.
***/ 

void
ehFndMsg A2(ulong, ulStampOff,
            ER,    Er)
{
   FILE *pFilMsg = NULL;
   char *pcSavPath = NULL;
   ulong ulTimeStamp;
   ulong ulSeekLoc;
   char  acMsgNam[EH_SZFILNAM+1];   /* file name */
   char  acMsgBuf[EH_MAXMSGLEN+1];  /* where the message is put */
   char *pcErrName = acMsgNam;

   // puts("ehFndMsg: entering");

   ehIndent(EHSTDERR);

   /*** initialization ***/

   sprintf(acMsgNam, EH_MSG_FORMAT, Er.acPrefix);

   if ((pFilMsg = fopen(acMsgNam, "r")) == NULL)
   {
      int   iPathLen;
      int   iNeedSlash;
      char *pcErrPaths;
      char *pcPath;

      if ((pcErrPaths = getenv("EHPATH")) == NULL)
      {
         // puts("ehFndMsg: fprintf 1");
         fprintf(EHSTDERR, "Can't open error message text file %s\n",
           acMsgNam);
         fflush(EHSTDERR);
         goto Exit;
      }

      /*** make a private buffer so we can muck with the path names ***/

      if ((pcSavPath = 
            (char *)calloc(strlen(pcErrPaths)+1, (size_t)1)) == NULL)
      {
         // puts("ehFndMsg: fprintf 2");
         fprintf(EHSTDERR, "calloc() failed\n");
         fflush(EHSTDERR);
         goto Exit;
      }
      pcErrPaths = strcpy(pcSavPath, pcErrPaths);

      while ((pcPath = strtok(pcErrPaths, 
#if ehNIX
            ":"
#else
            ";"
#endif
            )) != NULL)
      {
         pcErrPaths = NULL;

         iPathLen = (int)strlen(pcPath);
         iNeedSlash = (pcPath[iPathLen-1] != *pcEhSlash);
         if (pcErrName != acMsgNam) free(pcErrName); /* from last loop */
         if ((pcErrName = 
              (char*)calloc((size_t)(iPathLen + iNeedSlash + EH_SZFILNAM+1), 
               (size_t)1)) == NULL)
         {
            // puts("ehFndMsg: fprintf 3");
            fprintf(EHSTDERR, "calloc() failed\n");
            fflush(EHSTDERR);
            goto Exit;
         }
         sprintf(pcErrName, "%s%s%s", pcPath, iNeedSlash ?
               pcEhSlash : "", acMsgNam);
         assert((iPathLen + iNeedSlash + EH_SZFILNAM+1) > strlen(pcErrName));

         pFilMsg = fopen(pcErrName, "r");
         if (pFilMsg != NULL)
         {
            break;
         }
      }
      if (pFilMsg == NULL) /* fopen() failed for every try */
      {
         if (errno) 
         {
            // puts("ehFndMsg: fprintf 4");
            fprintf(EHSTDERR, "%s: %s\n", pcErrName, strerror(errno));
         }
         // puts("ehFndMsg: fprintf 5");
         fprintf(EHSTDERR, "Can't find error message text file %s\n",
               pcErrName);
         fflush(EHSTDERR);
         EHFREE (pcErrName);
         goto Exit;
      }
   }

   /*** verify time stamp ***/

   if (fscanf(pFilMsg, "0x%lx", &ulTimeStamp) != 1)
   {
      // puts("ehFndMsg: fprintf 6");
      fprintf(EHSTDERR, "fscanf() failed: bad format in %s\n",
            pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }
   if (ulTimeStamp != (ulStampOff & (-1L << OFFSET_WIDTH)))
   {
      // puts("ehFndMsg: fprintf 7");
      fprintf(EHSTDERR, "Error message file %s is not version 0x%lx\n",
            pcErrName, ulTimeStamp);
      ehIndent(EHSTDERR);
      fflush(EHSTDERR);
   }

   /*** fetch desired message from error-message text file ***/

   ulSeekLoc = ulStampOff & ~(-1L << OFFSET_WIDTH);
   if (fseek(pFilMsg, ulSeekLoc, SEEK_SET) != 0)
   {
      // puts("ehFndMsg: fprintf 8");
      fprintf(EHSTDERR, "Could not fseek() Error %s-%04x in file %s\n",
            Er.acPrefix, Er.hCode, pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }

   if (fgets(acMsgBuf, (int)sizeof(acMsgBuf), pFilMsg) == NULL)
   {
      // puts("ehFndMsg: fprintf 9");
      fprintf(EHSTDERR, "Error %s-%04x not found in file %s", 
           Er.acPrefix, Er.hCode, pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }
   assert(strlen(acMsgBuf) < sizeof(acMsgBuf));

   if (!isErrorMsg(acMsgBuf))
   {
      fprintf(EHSTDERR, "Bad error message format -- displaying anyway\n");
      ehIndent(EHSTDERR);
      fflush(EHSTDERR);
   }

   fprintf(EHSTDERR, "%s: ", Eh.pcProgName?Eh.pcProgName:"unknown");
   fprintf(EHSTDERR, "%s", acMsgBuf);
   fflush(EHSTDERR);
   if (EHSTDOUT != EHSTDERR)
   {
      fprintf(EHSTDOUT, "%s: ", Eh.pcProgName?Eh.pcProgName:"unknown");
      fprintf(EHSTDOUT, "%s", acMsgBuf);
      fflush(EHSTDOUT);
   }

Exit:   /* Exit Sequence */

   if (pcErrName != NULL && pcErrName != acMsgNam)
      free (pcErrName);

   if (pcSavPath)
      free(pcSavPath);

   if (pFilMsg)
      fclose(pFilMsg);

   // puts("ehFndMsg: leaving");
   return;
} /* ehFndMsg */


/***
ehLogTxt(pcLogTxt)
Append given error log text to the error log file.  This text will be 
prepended with relevent date/time/userid information.
The log text should have a newline.
***/

void
ehLogTxt A1(const char *, pcLogTxt)  /* text to be written */
{
   static int iLineNum = 0;
   pid_t iPid;
   const char *pcUserName;
   const char *pcNodeName;
   time_t TimeT;
   struct tm *pTM;
#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_PWD_H)
   pid_t iUid;
   struct utsname UTSName;
   struct passwd *pPassWd;

   iUid = getuid();
   pPassWd = getpwuid(iUid);
   pcUserName = pPassWd->pw_name;
   //puts("ehLogTxt: about to call uname()");
   uname(&UTSName);
   //puts("ehLogTxt: back from calling uname()");
   pcNodeName = UTSName.nodename;
   iPid = getpid();
#elif ehMSDOS
   pcUserName = "";
   pcNodeName = "MSDOS";
   iPid = 0;
#elif ehAMIGA
   pcUserName = "";
   pcNodeName = "Amiga";
   iPid = 0;
#else
   pcUserName = "";
   pcNodeName = "unknownhost";
   iPid = 0;
#endif

   // puts("ehLogTxt: entering");

   TimeT = time(NULL);
   pTM = localtime(&TimeT);

   ehIndent(EHSTDERR);
   fprintf(EHSTDERR, "[%04d %s@%s %24.24s %d] %s",
       iLineNum++, pcUserName, pcNodeName, asctime(pTM),
       iPid, pcLogTxt);
   fflush(EHSTDERR);

   if (iLineNum > 9999) 
      iLineNum = 0;

   //puts("ehLogTxt: leaving");
}


/***
ehLogFun()
Append given error log text to the error log file.
This text will be appended with relevent function information.
***/

void 
ehLogFun A6V(const char *, pcLogTxt,
             int,          iChan,
             int,          iVol,
             const char *, pcFile,
             const char *, pcFunc,
             int,          iLine,
             ...)
{
   va_list ap;

   //puts("ehLogFun: about to call VA_START");
   VA_START(ap, iLine); /* see eh.h */

   //puts("ehLogFun: about to call ehGetVol");
   if (ehGetVol(iChan) >= iVol)
   {
     ehIndent(EHSTDERR);
     fprintf(EHSTDERR, "%s: ", apcEhVol[iVol]);
     vfprintf(EHSTDERR, pcLogTxt, ap);
     fprintf(EHSTDERR, "<%s %s%s%d>\n", pcFile,
         pcFunc ? pcFunc : "",
         pcFunc ? "() "  : "",
         iLine);
     fflush(EHSTDERR);
   }
   //puts("ehLogFun: about to call va_end");
   va_end(ap);
}

/***
confirm format of error message
being read from error message file
***/

static
bool
isErrorMsg A1(
   char *, pcMsg)
{
   char acPrefix[EH_SZ_PREFIX+1];
   char acSuffix[32+1];
   char acMessage[128+1];
   int i;

   i = sscanf(pcMsg, "%[A-Z0-9]_%[A-Z0-9]: %[^\n]",
         acPrefix, acSuffix, acMessage);
   return (i == 3);
}

==> eh-0.5.tar.gz: eh-0.5/src/ehperror.c <==
/***

ehperror.c -- Error Handler, print error messages

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehperror.c,v 1.16 1996/05/12 15:48:30 ccobb Exp $";

/***

PrLNxSerror messages to the error log file.
Error messages are stored in error message files
having a name of the form:

   IDerrmsg.txt

These error message text files are genereated from an
error message "source" file with the utility ehdivsrc 
(error handler: divide source).  This utility takes an
error message source file and divides it into an error
message text file and a header file of the same name. 
The header file contains #defined ID's for each error 
message.  The appropriate ID must be passed to [UhS
EHTHROW(iErrorID) macro, which causes the appropriate 
error message to be fetched from the error message file
and written the error log file.

ehPError(iErrorID)

***/

#include "ehperror.h"

EHBEGINEXTERNC

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <time.h>

#if SUPPORTS_STDARG
#include <stdarg.h>
#else 
#include <varargs.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

EHENDEXTERNC

#include "ehchnvol.h"
#include "ehfuntrc.h"
#include "eherrlog.h"
//#include "ehpgmtrc.h"
#include "ehdivsrc.h"
#include "ehintini.h"
#include "ehercode.h"

static bool isErrorMsg A((char *pcMsg));

/***

ehPError()

Log error message according to error id.

ehPError(__FILE__, __FUNC__, __LINE__, scIsFatal, ulStampOff, Er);

CAUTION: This function may not return the SAME error code that was
         passed to it!  The reason for this is so the earliest/
         most severe error code is propogated to the completion
         of the program.  Therefore, if a severe error has already
         occurred and Eh.Er.u.hNo represents that fact, and this
         function is subsequently called with a less severe error
         code, then the more severe value of Eh.Er.u.hNo will be
         returned from this function rather than the error code
         that is passed in!

***/ 

void
ehPError A6(const char *, pcFilNam,
            const char *, pcFunNam,
            int,          iLineNum,
            schar,        scIsFatal, /* 1 = non=fatal, -1 = fatal */
            ulong,        ulStampOff,
            ulong,        ulHash)    /* "hashed" error code from error file */
{
   int   iSavErrNo = errno;
   ER Er = ehUnHash(ulHash);

   //printf("ehPError: ulHash = 0x%lx, Er is:\n", ulHash);
   //dmpER(Er);

   ehLogTxt("\n");   /* spit out system header stuff */

   /* output the error ID and location */
   ehIndent(EHSTDERR);
   fprintf(EHSTDERR, "error %s-%04d <%s %s%s%d>\n",
      Er.acPrefix, Er.hCode,
      pcFilNam,
      pcFunNam ? pcFunNam : "",
      pcFunNam ? "() "    : "",
      iLineNum);
   fflush(EHSTDERR);

   /* output the program name, error symbol & text */
   ehFndMsg(ulStampOff, Er); /* find message and send to stderr */

   if (iSavErrNo)
   {
      ehIndent(EHSTDERR);
      errno = iSavErrNo;
      fprintf(EHSTDERR, "errno: %s\n", strerror(errno));
      errno = 0;
   }
   ehCallStack(EHSTDERR);

   /*** now determine which error code to return ***/
   /*** return the earliest/most severe (first/worst) error ***/

   /* preserve the earliest FATAL error */;
   EhSt.scError = scIsFatal;  /* an error occurred in the current function */
   if (Eh.Er.hCode >= 0) /* a previous NON-fatal error */
   {
      strcpy(Eh.Er.acPrefix, Er.acPrefix);
      if (scIsFatal == -1) /* override previous NON-fatal error w fatal error */
      {
         Eh.Er.hCode = (short)-Er.hCode; /* set to fatal error */
      }
      else if (Eh.Er.hCode == 0)  /* don't over-write previous error */
      {
         Eh.Er.hCode =  Er.hCode;
      }
   }
   //printf("ehPError: Eh.Er.lNo   = 0x%x\n", Eh.Er.lNo);
   //printf("ehPError: Eh.Er.u.c0  = %c\n",   Eh.Er.u.c0);
   //printf("ehPError: Eh.Er.u.c1  = %c\n",   Eh.Er.u.c1);
   //printf("ehPError: Eh.Er.u.hNo = %d\n",   Eh.Er.u.hNo);
   if (scIsFatal == -1)
   {
      EhSt.yChain = 1;   /* set the chaining bit */

      /***
         NOTE:
         The following are NOT guarenteed to
         cause a longjmp or throw.  We may not
         be allowed to jump/throw now, in which
         case we will just return.
      ***/

#ifdef ehEXCEPTIONHANDLING

      ehPErrorThrow(pcFunNam);   /* throw ! */

#else /* !defined(__cplusplus) || !ehEXCEPTIONHANDLING */

      ehFatalJmp();              /* longjmp() ! */

#endif /* !defined(__cplusplus) | !ehEXCEPTIONHANDLING */

   }
   return;
} /* ehPError */


/***
ehFndMsg()
Error Handler -- Find error message and send to stderr.
Terminate with a newline.
***/ 

void
ehFndMsg A2(ulong, ulStampOff,
            ER,    Er)
{
   FILE *pFilMsg = NULL;
   char *pcSavPath = NULL;
   ulong ulTimeStamp;
   ulong ulSeekLoc;
   char  acMsgNam[EH_SZFILNAM+1];   /* file name */
   char  acMsgBuf[EH_MAXMSGLEN+1];  /* where the message is put */
   char *pcErrName = acMsgNam;

   // puts("ehFndMsg: entering");

   ehIndent(EHSTDERR);

   /*** initialization ***/

   sprintf(acMsgNam, EH_MSG_FORMAT, Er.acPrefix);

   if ((pFilMsg = fopen(acMsgNam, "r")) == NULL)
   {
      int   iPathLen;
      int   iNeedSlash;
      char *pcErrPaths;
      char *pcPath;

      if ((pcErrPaths = getenv("EHPATH")) == NULL)
      {
         // puts("ehFndMsg: fprintf 1");
         fprintf(EHSTDERR, "Can't open error message text file %s\n",
           acMsgNam);
         fflush(EHSTDERR);
         goto Exit;
      }

      /*** make a private buffer so we can muck with the path names ***/

      if ((pcSavPath = 
            (char *)calloc(strlen(pcErrPaths)+1, (size_t)1)) == NULL)
      {
         // puts("ehFndMsg: fprintf 2");
         fprintf(EHSTDERR, "calloc() failed\n");
         fflush(EHSTDERR);
         goto Exit;
      }
      pcErrPaths = strcpy(pcSavPath, pcErrPaths);

      while ((pcPath = strtok(pcErrPaths, 
#if ehNIX
            ":"
#else
            ";"
#endif
            )) != NULL)
      {
         pcErrPaths = NULL;

         iPathLen = (int)strlen(pcPath);
         iNeedSlash = (pcPath[iPathLen-1] != *pcEhSlash);
         if (pcErrName != acMsgNam) free(pcErrName); /* from last loop */
         if ((pcErrName = 
              (char*)calloc((size_t)(iPathLen + iNeedSlash + EH_SZFILNAM+1), 
               (size_t)1)) == NULL)
         {
            // puts("ehFndMsg: fprintf 3");
            fprintf(EHSTDERR, "calloc() failed\n");
            fflush(EHSTDERR);
            goto Exit;
         }
         sprintf(pcErrName, "%s%s%s", pcPath, iNeedSlash ?
               pcEhSlash : "", acMsgNam);
         assert((iPathLen + iNeedSlash + EH_SZFILNAM+1) > strlen(pcErrName));

         pFilMsg = fopen(pcErrName, "r");
         if (pFilMsg != NULL)
         {
            break;
         }
      }
      if (pFilMsg == NULL) /* fopen() failed for every try */
      {
         if (errno) 
         {
            // puts("ehFndMsg: fprintf 4");
            fprintf(EHSTDERR, "%s: %s\n", pcErrName, strerror(errno));
         }
         // puts("ehFndMsg: fprintf 5");
         fprintf(EHSTDERR, "Can't find error message text file %s\n",
               pcErrName);
         fflush(EHSTDERR);
         if (EHSTDOUT != EHSTDERR)
         {
            fprintf(EHSTDOUT, "Can't find error message text file %s\n",
               pcErrName);
            fprintf(EHSTDOUT, "Check your $EHPATH setting.\n");
            fflush(EHSTDOUT);
         }
         EHFREE (pcErrName);
         goto Exit;
      }
   }

   /*** verify time stamp ***/

   if (fscanf(pFilMsg, "0x%lx", &ulTimeStamp) != 1)
   {
      // puts("ehFndMsg: fprintf 6");
      fprintf(EHSTDERR, "fscanf() failed: bad format in %s\n",
            pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }
   if (ulTimeStamp != (ulStampOff & (-1L << OFFSET_WIDTH)))
   {
      // puts("ehFndMsg: fprintf 7");
      fprintf(EHSTDERR, "Error message file %s is not version 0x%lx\n",
            pcErrName, ulTimeStamp);
      ehIndent(EHSTDERR);
      fflush(EHSTDERR);
   }

   /*** fetch desired message from error-message text file ***/

   ulSeekLoc = ulStampOff & ~(-1L << OFFSET_WIDTH);
   if (fseek(pFilMsg, ulSeekLoc, SEEK_SET) != 0)
   {
      // puts("ehFndMsg: fprintf 8");
      fprintf(EHSTDERR, "Could not fseek() Error %s-%04d in file %s\n",
            Er.acPrefix, Er.hCode, pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }

   if (fgets(acMsgBuf, (int)sizeof(acMsgBuf), pFilMsg) == NULL)
   {
      // puts("ehFndMsg: fprintf 9");
      fprintf(EHSTDERR, "Error %s-%04d not found in file %s", 
           Er.acPrefix, Er.hCode, pcErrName);
      fflush(EHSTDERR);
      goto Exit;
   }
   assert(strlen(acMsgBuf) < sizeof(acMsgBuf));

   if (!isErrorMsg(acMsgBuf))
   {
      fprintf(EHSTDERR, "Bad error message format -- displaying anyway\n");
      ehIndent(EHSTDERR);
      fflush(EHSTDERR);
   }

   fprintf(EHSTDERR, "%s: ", Eh.pcProgName?Eh.pcProgName:"unknown");
   fprintf(EHSTDERR, "%s", acMsgBuf);
   fflush(EHSTDERR);
   if (EHSTDOUT != EHSTDERR)
   {
      fprintf(EHSTDOUT, "%s: ", Eh.pcProgName?Eh.pcProgName:"unknown");
      fprintf(EHSTDOUT, "%s", acMsgBuf);
      fflush(EHSTDOUT);
   }

Exit:   /* Exit Sequence */

   if (pcErrName != NULL && pcErrName != acMsgNam)
      free (pcErrName);

   if (pcSavPath)
      free(pcSavPath);

   if (pFilMsg)
      fclose(pFilMsg);

   // puts("ehFndMsg: leaving");
   return;
} /* ehFndMsg */


/***
ehLogTxt(pcLogTxt)
Append given error log text to the error log file.  This text will be 
prepended with relevent date/time/userid information.
The log text should have a newline.
***/

void
ehLogTxt A1(const char *, pcLogTxt)  /* text to be written */
{
   static int iLineNum = 0;
   pid_t iPid;
   const char *pcUserName;
   const char *pcNodeName;
   time_t TimeT;
   struct tm *pTM;
#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_PWD_H)
   pid_t iUid;
   struct utsname UTSName;
   struct passwd *pPassWd;

   iUid = getuid();
   pPassWd = getpwuid(iUid);
   pcUserName = pPassWd->pw_name;
   //puts("ehLogTxt: about to call uname()");
   uname(&UTSName);
   //puts("ehLogTxt: back from calling uname()");
   pcNodeName = UTSName.nodename;
   iPid = getpid();
#elif ehMSDOS
   pcUserName = "";
   pcNodeName = "MSDOS";
   iPid = 0;
#elif ehAMIGA
   pcUserName = "";
   pcNodeName = "Amiga";
   iPid = 0;
#else
   pcUserName = "";
   pcNodeName = "unknownhost";
   iPid = 0;
#endif

   // puts("ehLogTxt: entering");

   TimeT = time(NULL);
   pTM = localtime(&TimeT);

   ehIndent(EHSTDERR);
   fprintf(EHSTDERR, "[%04d %s@%s %24.24s %d] %s",
       iLineNum++, pcUserName, pcNodeName, asctime(pTM),
       iPid, pcLogTxt);
   fflush(EHSTDERR);

   if (iLineNum > 9999) 
      iLineNum = 0;

   //puts("ehLogTxt: leaving");
}


/***
ehLogFun()
Append given error log text to the error log file.
This text will be appended with relevent function information.
***/

void 
ehLogFun A6V(const char *, pcLogTxt,
             int,          iChan,
             int,          iVol,
             const char *, pcFile,
             const char *, pcFunc,
             int,          iLine,
             ...)
{
   va_list ap;

   //puts("ehLogFun: about to call VA_START");
   VA_START(ap, iLine); /* see eh.h */

   //puts("ehLogFun: about to call ehGetVol");
   if (ehGetVol(iChan) >= iVol)
   {
     ehIndent(EHSTDERR);
     fprintf(EHSTDERR, "%s: ", apcEhVol[iVol]);
     vfprintf(EHSTDERR, pcLogTxt, ap);
     fprintf(EHSTDERR, "<%s %s%s%d>\n", pcFile,
         pcFunc ? pcFunc : "",
         pcFunc ? "() "  : "",
         iLine);
     fflush(EHSTDERR);
   }
   //puts("ehLogFun: about to call va_end");
   va_end(ap);
}

/***
confirm format of error message
being read from error message file
***/

static
bool
isErrorMsg A1(
   char *, pcMsg)
{
   char acPrefix[EH_SZ_PREFIX+1];
   char acSuffix[32+1];
   char acMessage[128+1];
   int i;

   i = sscanf(pcMsg, "%[A-Z0-9]_%[A-Z0-9]: %[^\n]",
         acPrefix, acSuffix, acMessage);
   return (i == 3);
}
